'''
Created on 09/07/2014

@author: synerty
'''
from peek_server.storage import dbConn
from peek_server.storage.PeekEnv import PeekEnvServer, PeekEnvAgent, PeekEnvWorker, \
    PeekEnvClient
from vortex.sqla_orm.OrmCrudHandler import OrmCrudHandler, OrmCrudHandlerExtension

# -----------------------------------------------------------------------------
# Servers
serverListDataKey = {
    'plugin': 'peek_server',
    'key': "peakadm.env.server.list.data"
}


class EnvServerListHandler(OrmCrudHandler):
    pass


envServerListHandler = EnvServerListHandler(dbConn.ormSessionCreator,
                                            PeekEnvServer,
                                            serverListDataKey,
                                            retreiveAll=True)


@envServerListHandler.addExtension(PeekEnvServer)
class _EnvServerListHandlerExtension(OrmCrudHandlerExtension):
    def afterUpdateCommit(self, tuple_, tuples, session, payloadFilt):
        # from peek.api.client.ClientGridHandler import clientGridHandler
        # reactor.callLater(0.0, clientGridHandler.reload)
        return True


# -----------------------------------------------------------------------------
# workers
workerListDataKey = {
    'plugin': 'peek_server',
    'key': "peakadm.env.worker.list.data"
}


class EnvWorkerListHandler(OrmCrudHandler):
    pass


envWorkerListHandler = EnvWorkerListHandler(dbConn.ormSessionCreator,
                                            PeekEnvWorker,
                                            workerListDataKey,
                                            retreiveAll=True)


@envWorkerListHandler.addExtension(PeekEnvWorker)
class _EnvWorkerListHandlerExtension(OrmCrudHandlerExtension):
    def afterUpdateCommit(self, tuple_, tuples, session, payloadFilt):
        # from peek.api.client.ClientGridHandler import clientGridHandler
        # reactor.callLater(0.0, clientGridHandler.reload)
        return True

# -----------------------------------------------------------------------------
# agents
agentListDataKey = {
    'plugin': 'peek_server',
    'key': "peakadm.env.agent.list.data"
}

class EnvAgentListHandler(OrmCrudHandler):
    pass

envAgentListHandler = EnvAgentListHandler(dbConn.ormSessionCreator,
                                          PeekEnvAgent,
                                          agentListDataKey,
                                          retreiveAll=True)

@envAgentListHandler.addExtension(PeekEnvAgent)
class _EnvAgentListHandlerExtension(OrmCrudHandlerExtension):
    def afterUpdateCommit(self, tuple_, tuples, session, payloadFilt):
        return True

# -----------------------------------------------------------------------------
# agents
clientListDataKey = {
    'plugin': 'peek_server',
    'key': "peakadm.env.client.list.data"
}


class EnvClientListHandler(OrmCrudHandler):
    pass


envClientListHandler = EnvClientListHandler(dbConn.ormSessionCreator,
                                          PeekEnvClient,
                                           clientListDataKey,
                                          retreiveAll=True)


@envClientListHandler.addExtension(PeekEnvClient)
class _EnvClientListHandlerExtension(OrmCrudHandlerExtension):
    def afterUpdateCommit(self, tuple_, tuples, session, payloadFilt):
        return True



"""Added Dxf model

Revision ID: 5ad8b58df646
Revises: 
Create Date: 2015-06-02 22:08:03.996137

"""

# revision identifiers, used by Alembic.
revision = '5ad8b58df646'
down_revision = None
branch_labels = None
depends_on = None



from alembic import op
import sqlalchemy as sa
import geoalchemy2
from sqlalchemy.dialects import postgresql



def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###



'''
Created on 09/07/2014

@author: synerty
'''
from peek_server.server.sw_version.PluginSwVersionInfoUtil import getLatestPluginVersionInfos
from txhttputil.util.DeferUtil import deferToThreadWrap
from vortex.handler.ModelHandler import ModelHandler

pluginVersionInfoHandlerKey = {
    'plugin': 'peek_server',
    'key': "peek_server.plugin.version.info"
}


class __PluginVersionInfoHandler(ModelHandler):
    @deferToThreadWrap
    def buildModel(self, payloadFilt, **kwargs):
        return getLatestPluginVersionInfos()


__pluginVersionInfoHandler = __PluginVersionInfoHandler(pluginVersionInfoHandlerKey)



"""
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
import logging
import os

from twisted.web.server import NOT_DONE_YET
from txhttputil.site.BasicResource import BasicResource
from txhttputil.site.StaticFileResource import StaticFileResource

from peek_platform import PeekPlatformConfig
from peek_platform.sw_install.PeekSwInstallManagerABC import PeekSwInstallManagerABC

logger = logging.getLogger(__name__)


class PeekSwUpdateDownloadResource(BasicResource):
    isLeaf = True
    isGzipped = True

    def render_GET(self, request):
        componentName = request.args.get(b'name', [None])[0]
        version = request.args.get(b'version', [None])[0]

        if not componentName and not version:
            msg = "Download requires name and version, Name=%s, Version=%s"
            msg %= (componentName, version)
            logger.error(msg)
            request.write(msg.encode())
            request.finish()
            return NOT_DONE_YET

        componentName = componentName.decode()
        version = version.decode()

        logger.debug("Peek Platform Download Resource GET, name=%s, version=%s",
                     componentName, version)

        newSoftwareTar = PeekSwInstallManagerABC.makeReleaseFileName(version)

        request.responseHeaders.setRawHeaders(b'content-type',
                                              [b'application/octet-stream'])

        resource = StaticFileResource(newSoftwareTar)
        return resource.render_GET(request)





'''
Created on 09/07/2014

@author: synerty
'''
from peek_platform import PeekPlatformConfig
from peek_server.backend.navbar.PeekAdmNavbarUserTuple import PeekAdmNavbarUserTuple
from vortex.handler.ModelHandler import ModelHandler

navbarDataFiltKey = {
    'plugin': 'peek_server',
    'key': "nav.adm.user.data"
}


# HANDLER
class __NavbarHandler(ModelHandler):
    def buildModel(self,
                   # payloadFilt=None,
                   # vortexUuid=None,
                   # payload=Payload(),
                   # session=None,
                   **kwargs):

        navbarTuple = PeekAdmNavbarUserTuple()
        lic = PeekPlatformConfig.config.capabilities

        navbarTuple.supportExceeded = lic['supportExceeded'] and not lic['demoExceeded']
        navbarTuple.demoExceeded = lic['demoExceeded']
        navbarTuple.countsExceeded = lic['countsExceeded']
        navbarTuple.username = "None"  # userAccess.username

        return [navbarTuple]


navbarDataHandler = __NavbarHandler(navbarDataFiltKey)



""" 
 * view.common.uiobj.Style.py
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by 
 *  Synerty Pty Ltd
 *
"""
from peek_server.ui_main.login import AppLoginElement
from twisted.web._flatten import flattenString
from twisted.web.resource import Resource
from twisted.web.server import NOT_DONE_YET

from txhttputil import ElementRenderedCallback


class LoginResource(Resource):
  isLeaf = True

  def render_GET(self, request):
    return self._renderLogin(request)

  def render_POST(self, request):
    return self._renderLogin(request, failed=True)

  def _renderLogin(self, request, failed=False):
    request.responseHeaders.setRawHeaders("authorization", ["basic"])

    callback = ElementRenderedCallback(request).elementRenderedCallback

    request.write('<!DOCTYPE html>\n')

    d = flattenString(request, AppLoginElement(failed=failed))
    d.addBoth(callback)
    return NOT_DONE_YET

class LoginSucceededResource(Resource):
  isLeaf = True

  def render_GET(self, request):
    return self._render(request)

  def render_POST(self, request):
    return self._render(request)

  def _render(self, request):
    request.redirect(request.uri)
    request.finish()
    return NOT_DONE_YET



from twisted.cred import error
from twisted.python.components import registerAdapter, proxyForInterface
from twisted.web._auth.wrapper import UnauthorizedResource
from twisted.web.resource import IResource, ErrorPage
from twisted.web.server import Session
from twisted.web.util import DeferredResource
from zope.interface import Interface, Attribute, implements

from txhttputil import AuthUserDetails
from txhttputil import createRootResource
from .AuthResource import LoginResource, LoginSucceededResource

__author__ = 'synerty'


class IAuth(Interface):
  value = Attribute("An int value which counts up once per page view.")


class Auth(object):
  implements(IAuth)

  def __init__(self, session):
    self.userAccess = AuthUserDetails()
    self.userAccess.loggedIn = False
    self.userAccess.readOnly = True


registerAdapter(Auth, Session, IAuth)


class AuthSessionWrapper(object):
  implements(IResource)
  isLeaf = False


  def _authorizedResource(self, request):
    """
    Get the L{IResource} which the given request is authorized to receive.
    If the proper authorization headers are present, the resource will be
    requested from the portal.  If not, an anonymous login_page attempt will be
    made.
    """

    auth = IAuth(request.getSession())
    if auth.userAccess.loggedIn:
      request.getSession().touch()
      return createRootResource(auth.userAccess)

    # TRY Basic HTTP Authentication
    authheader = request.getHeader('authorization')
    if authheader:
      factory, respString = self._selectParseHeader(authheader)
      if factory is None:
        return LoginResource()
      try:
        credentials = factory.decode(respString, request)
      except error.LoginFailed:
        return LoginResource()
      except:
        print("Unexpected failure from credentials factory")
        return ErrorPage(500, None, None)
      else:
        return DeferredResource(self._login(credentials))


    # TRY form based authentication
    a = request.args
    if request.method == "POST":
      if ("username" in a
          and "password" in a
          and a['username'][0] == "test"
          and a["password"][0] == "test"):
        auth.userAccess.loggedIn = True
        auth.userAccess.readOnly = False
        request.redirect(request.path)
        return LoginSucceededResource()

    # Return login_page form
    return LoginResource()


  def render(self, request):
    """
    Find the L{IResource} avatar suitable for the given request, if
    possible, and render it.  Otherwise, perhaps render an error page
    requiring authorization or describing an internal server failure.
    """
    return self._authorizedResource(request).render(request)


  def getChildWithDefault(self, path, request):
    """
    Inspect the Authorization HTTP header, and return a deferred which,
    when fired after successful authentication, will return an authorized
    C{Avatar}. On authentication failure, an C{UnauthorizedResource} will
    be returned, essentially halting further dispatch on the wrapped
    resource and all children
    """
    # Don't consume any segments of the request - this class should be
    # transparent!
    request.postpath.insert(0, request.prepath.pop())
    return self._authorizedResource(request)


  def _login(self, credentials):
    """
    Get the L{IResource} avatar for the given credentials.

    @return: A L{Deferred} which will be called back with an L{IResource}
        avatar or which will errback if authentication fails.
    """
    d = self._portal.login(credentials, None, IResource)
    d.addCallbacks(self._loginSucceeded, self._loginFailed)
    return d


  def _loginSucceeded(self, xxx_todo_changeme):
    """
    Handle login_page success by wrapping the resulting L{IResource} avatar
    so that the C{logout} callback will be invoked when rendering is
    complete.
    """
    (interface, avatar, logout) = xxx_todo_changeme
    class ResourceWrapper(proxyForInterface(IResource, 'resource')):
      """
      Wrap an L{IResource} so that whenever it or a child of it
      completes rendering, the cred logout hook will be invoked.

      An assumption is made here that exactly one L{IResource} from
      among C{avatar} and all of its children will be rendered.  If
      more than one is rendered, C{logout} will be invoked multiple
      times and probably earlier than desired.
      """

      def getChildWithDefault(self, name, request):
        """
        Pass through the lookup to the wrapped resource, wrapping
        the result in L{ResourceWrapper} to ensure C{logout} is
        called when rendering of the child is complete.
        """
        return ResourceWrapper(self.resource.getChildWithDefault(name, request))

      def render(self, request):
        """
        Hook into response generation so that when rendering has
        finished completely (with or without error), C{logout} is
        called.
        """
        request.notifyFinish().addBoth(lambda ign: logout())
        return super(ResourceWrapper, self).render(request)

    return ResourceWrapper(avatar)


  def _loginFailed(self, result):
    """
    Handle login_page failure by presenting either another challenge (for
    expected authentication/authorization-related failures) or a server
    error page (for anything else).
    """
    if result.check(error.Unauthorized, error.LoginFailed):
      return UnauthorizedResource(self._credentialFactories)
    else:
      print()
      result,
      "HTTPAuthSessionWrapper.getChildWithDefault encountered "
      "unexpected error"
    return ErrorPage(500, None, None)


  def _selectParseHeader(self, header):
    """
    Choose an C{ICredentialFactory} from C{_credentialFactories}
    suitable to use to decode the given I{Authenticate} header.

    @return: A two-tuple of a factory and the remaining portion of the
        header value to be decoded or a two-tuple of C{None} if no
        factory can decode the header value.
    """
    elements = header.split(' ')
    scheme = elements[0].lower()
    for fact in self._credentialFactories:
      if fact.scheme == scheme:
        return (fact, ' '.join(elements[1:]))
    return (None, None)



"""Added MSSQL csvtotable

Revision ID: c2a3b93e178d
Revises: 05cf60a0fd22
Create Date: 2017-07-17 10:48:07.127510

"""

# revision identifiers, used by Alembic.
from sqlalchemy.dialects.mssql.base import MSDialect
from sqlalchemy.dialects.postgresql.base import PGDialect

revision = 'c2a3b93e178d'
down_revision = '05cf60a0fd22'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2

def isMssqlDialect():
    return isinstance(op.get_bind().engine.dialect, MSDialect)


def isPostGreSQLDialect():
    return isinstance(op.get_bind().engine.dialect, PGDialect)


def upgrade():
    msSqlForVarchar = '''
        CREATE FUNCTION [dbo].[peekCsvVarcharToTable](@input AS Varchar(max) )
        RETURNS
              @Result TABLE(Value varchar(100))
        AS
        BEGIN
              DECLARE @str VARCHAR(20)
              DECLARE @ind Int
              IF(@input is not null)
              BEGIN
                    SET @ind = CharIndex(',',@input)
                    WHILE @ind > 0
                    BEGIN
                          SET @str = SUBSTRING(@input,1,@ind-1)
                          SET @input = SUBSTRING(@input,@ind+1,LEN(@input)-@ind)
                          INSERT INTO @Result values (@str)
                          SET @ind = CharIndex(',',@input)
                    END
                    SET @str = @input
                    INSERT INTO @Result values (@str)
              END
              RETURN
        END
    '''

    msSqlForInt = '''
        CREATE FUNCTION [dbo].[peekCsvIntToTable](@input AS Varchar(max) )
        RETURNS
              @Result TABLE(Value bigint)
        AS
        BEGIN
              DECLARE @str VARCHAR(20)
              DECLARE @ind Int
              IF(@input is not null)
              BEGIN
                    SET @ind = CharIndex(',',@input)
                    WHILE @ind > 0
                    BEGIN
                          SET @str = SUBSTRING(@input,1,@ind-1)
                          SET @input = SUBSTRING(@input,@ind+1,LEN(@input)-@ind)
                          INSERT INTO @Result values (@str)
                          SET @ind = CharIndex(',',@input)
                    END
                    SET @str = @input
                    INSERT INTO @Result values (@str)
              END
              RETURN
        END
    '''

    if isMssqlDialect():
        op.execute(msSqlForVarchar)
        op.execute(msSqlForInt)


def downgrade():
    raise NotImplementedError()



import os
import shutil
from setuptools import setup

pip_package_name = "peek-server"
py_package_name = "peek_server"

package_version = '0.7.9.3'

egg_info = "%s.egg-info" % pip_package_name
if os.path.isdir(egg_info):
    shutil.rmtree(egg_info)

if os.path.isfile('MANIFEST'):
    os.remove('MANIFEST')

excludePathContains = ('__pycache__', 'node_modules', 'platforms', 'dist')
excludeFilesEndWith = ('.pyc', '.js', '.js.map', '.lastHash')
excludeFilesStartWith = ()


def find_package_files():
    paths = []
    for (path, directories, filenames) in os.walk(py_package_name):
        if [e for e in excludePathContains if e in path]:
            continue

        for filename in filenames:
            if [e for e in excludeFilesEndWith if filename.endswith(e)]:
                continue

            if [e for e in excludeFilesStartWith if filename.startswith(e)]:
                continue

            paths.append(os.path.join(path[len(py_package_name) + 1:], filename))

    return paths

package_files = find_package_files()

setup(
    name=pip_package_name,
    packages=[py_package_name],
    package_data={'': package_files},
    entry_points={
        'console_scripts': [
            'run_peek_server = peek_server.run_peek_server:main',
            'winsvc_peek_server = peek_server.winsvc_peek_server:main',
        ],
    },
    install_requires=["peek-platform", "peek-admin"],
    zip_safe=False,version=package_version,
    description='Peek Platform - Server Service',
    author='Synerty',
    author_email='contact@synerty.com',
    url='https://github.com/Synerty/%s' % pip_package_name,
    download_url='https://github.com/Synerty/%s/tarball/%s' % (
        pip_package_name, package_version),
    keywords=['Peek', 'Python', 'Platform', 'synerty'],
    classifiers=[
        "Programming Language :: Python :: 3.5",
    ],
)



from pathlib import Path

from peek_platform import PeekPlatformConfig
from peek_plugin_base.server.PeekPlatformServerHttpHookABC import PeekPlatformServerHttpHookABC
from peek_plugin_base.server.PeekPlatformAdminHttpHookABC import PeekPlatformAdminHttpHookABC
from peek_plugin_base.server.PeekServerPlatformHookABC import PeekServerPlatformHookABC


class PeekServerPlatformHook(PeekServerPlatformHookABC):
    def __init__(self, pluginName: str):
        PeekPlatformAdminHttpHookABC.__init__(self)
        PeekPlatformServerHttpHookABC.__init__(self)
        self._pluginName = pluginName

    @property
    def dbConnectString(self) -> str:
        from peek_platform import PeekPlatformConfig
        return PeekPlatformConfig.config.dbConnectString

    @property
    def fileStorageDirectory(self) -> Path:
        from peek_platform import PeekPlatformConfig

        return Path(PeekPlatformConfig.config.pluginDataPath(self._pluginName))

    def _getOtherPluginApi(self, pluginName):
        serverPluginLoader = PeekPlatformConfig.pluginLoader

        otherPlugin = serverPluginLoader.pluginEntryHook(pluginName)
        if not otherPlugin:
            return None

        from peek_plugin_base.server.PluginServerEntryHookABC import \
            PluginServerEntryHookABC

        assert isinstance(otherPlugin, PluginServerEntryHookABC), (
            "Not an instance of PluginServerEntryHookABC")

        return otherPlugin

    def getOtherPluginApi(self, pluginName):
        otherPlugin = self._getOtherPluginApi(pluginName)
        return otherPlugin.publishedServerApi if otherPlugin else None

    @property
    def serviceId(self) -> str:
        import socket
        return "server|" + socket.gethostname()
